#!/usr/bin/perl 

# my_favorites.cgi v1.0 (3.24.2006)
# this file reads the cookie ContentDM sets for a users chosen favorites	
# and collects all of those favorites into a zip file for download.
#
# Copyright Matt Bockol 2006 mbockol@carleton.edu
#	
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



use CGI qw/:standard/; 
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use Data::Dumper ; 
use utf8;
use strict ; 

# the location ContentDM stores the index of all collections
my $catalog = '/usr/local/Content/docs/catalog.txt';

# a hash of the collections and their paths on the filesystem
my %collection_map = %{parse_catalog($catalog)}; 

# a hash of the collections and their image id to filename $collection_index{collection}{id} = filename
my %collection_index = %{parse_indicies(\%collection_map)};

# pull the user's cookie collecting their favorites
my $favorites_flat = cookie('BUF');
if(!defined($favorites_flat)){ $favorites_flat = ""; }

# clean up the data for easier parsing
$favorites_flat =~ s/^\///g ; 
my @favorites = split(/\//, $favorites_flat); 


# create a new zipfile we can place images into
my $zip = Archive::Zip->new();

# interate over the favorites, adding each to the zip
foreach(@favorites){ 

	my $fav_line = $_ ; 

	$fav_line =~ s/>$//g ; 

	my ($collection , $item) = split(/</, $fav_line); 

	my $image_path = $collection_map{$collection} . "/image/" . $collection_index{$collection}{$item} ;

	# only try to include it if the file exists
	if(-e $image_path){
		$zip->addFile($image_path , $collection . "-" . $collection_index{$collection}{$item}); 
		}

	}


# pick a random zipfile name, make sure it doesn't already exist
my $tmp_zipfile = "/tmp/cdm-" . generate_random_string(10) . ".zip" ; 
while(-e $tmp_zipfile){ $tmp_zipfile = "/tmp/cdm-" . generate_random_string(10) . ".zip" ; }

# write the zipfile out to the file
die 'write error' unless $zip->writeToFileNamed( $tmp_zipfile ) == AZ_OK;

# if the zip file exists, send it to the user
if(-e $tmp_zipfile){

	# send the header renaming the file to something the user would expect
	print header(-type=>'application/zip',-attachment=>'my_favorites.zip');

	unless(open(ZIPFILE , $tmp_zipfile)){ die("unable to open $tmp_zipfile"); }
	while(<ZIPFILE>){ print $_ ; }
	close(ZIPFILE);

	}
else { 
	print header ; 
	print "An error has occured. Please contact the server administrator.";
	die();
	}


# clean up after ourselves
unlink($tmp_zipfile);


# determine where collection names reside on the filesystem
sub parse_catalog { 
	my $catalog_file = shift ; 

	my %name_path ; 

	unless(open(CAT , $catalog_file)){
		print header();
		die("Unable to open the catalog file."); 
		}

	while(<CAT>){
		my $line = $_ ; 

		if($line =~ /^\// ){
			# file format: collection name\t\tlong collection name\tcollection path
			$line =~ /(.*)\t\t(.*)\t(.*)/ ; 
			my $short = $1 ; 
			my $long = $2 ; 
			my $path = $3 ; 
			
			$short =~ s/^\///g ; 
			$name_path{$short} = $path ; 
			}
		}

	close(CAT);

	return \%name_path ; 

	}

# entries are stored all on a single line in a utf8 text file with a new entry every 160 characters (not 160 bytes!)
sub parse_indicies { 
	my $collection_map_ref = shift ; 
	my %collection_map = %{$collection_map_ref} ; 

	my %collection_index ; 

	foreach(keys %collection_map){ 
		my $collection = $_ ; 
		my $entry_index_file = $collection_map{$collection} . "/index/text_search/entry.index" ; 

		# gather ye filenames as ye may	
		my $line = ""; 
	
		# only index entry files which exist :)	
		if(-e $entry_index_file){
			unless(open(ENTRY , $entry_index_file)){ print header(); die("Unable to open the catalog file $entry_index_file."); }
			while(<ENTRY>){ $line = $_ ; }
			close(ENTRY);
			}

		# ContentDM stores this data in UTF8, perl needs to be told or the substr function will
		# treat unicode characters as two and break the offset calculation
		utf8::encode($line);
		utf8::upgrade($line);
		utf8::decode($line);
		
		my $increment = 160 ; 
		my $position = 0 ; 


		while($position <= length($line)){
			my $item_line = substr($line , $position, $increment) ;
			$item_line =~ s/\s+/ /g ; 		

			my ($id , $file , @other) = split(/\s/ , $item_line); 

			if(defined($id) && defined($file)){
				$collection_index{$collection}{$id} = $file ; 
				}	
	
			$position += $increment ; 
			}
		}


	return \%collection_index ; 

	}


# for tmp zip file filename
sub generate_random_string
{
	my $length_of_randomstring=shift;# the length of 
			 # the random string to generate

	my @chars=('a'..'z','A'..'Z','0'..'9','_');
	my $random_string;
	foreach (1..$length_of_randomstring) 
	{
		# rand @chars will generate a random 
		# number between 0 and scalar @chars
		$random_string.=$chars[rand @chars];
	}
	return $random_string;
}
